import { useReducer, useEffect, useRef } from 'react';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

var __assign = Object.assign || function __assign(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }
    return t;
};

var defaultState = function (defaultValue) {
    return {
        loading: defaultValue === undefined || defaultValue === null,
        value: defaultValue,
    };
};
var reducer = function (state, action) {
    switch (action.type) {
        case 'error':
            return __assign({}, state, { error: action.error, loading: false });
        case 'reset':
            return defaultState(action.defaultValue);
        case 'value':
            return __assign({}, state, { loading: false, value: action.value });
        default:
            return state;
    }
};
var useLoadingValue = (function (getDefaultValue) {
    var defaultValue = getDefaultValue ? getDefaultValue() : undefined;
    var _a = useReducer(reducer, defaultState(defaultValue)), state = _a[0], dispatch = _a[1];
    var reset = function () {
        var defaultValue = getDefaultValue ? getDefaultValue() : undefined;
        dispatch({ type: 'reset', defaultValue: defaultValue });
    };
    var setError = function (error) {
        dispatch({ type: 'error', error: error });
    };
    var setValue = function (value) {
        dispatch({ type: 'value', value: value });
    };
    return {
        error: state.error,
        loading: state.loading,
        reset: reset,
        setError: setError,
        setValue: setValue,
        value: state.value,
    };
});

var useComparatorRef = function (value, isEqual, onChange) {
    var ref = useRef(value);
    useEffect(function () {
        if (!isEqual(value, ref.current)) {
            ref.current = value;
            if (onChange) {
                onChange();
            }
        }
    });
    return ref;
};
var useIsEqualRef = function (value, onChange) {
    function isEqual(v1, v2) {
        var bothNull = !v1 && !v2;
        var equal = !!v1 && !!v2 && v1.isEqual(v2);
        return bothNull || equal;
    }
    return useComparatorRef(value, isEqual, onChange);
};

var initialState = {
    loading: true,
    value: {
        keys: [],
        values: [],
    },
};
var reducer$1 = function (state, action) {
    switch (action.type) {
        case 'add':
            if (!action.snapshot) {
                return state;
            }
            return __assign({}, state, { value: addChild(state.value, action.snapshot, action.previousKey) });
        case 'change':
            if (!action.snapshot) {
                return state;
            }
            return __assign({}, state, { value: changeChild(state.value, action.snapshot) });
        case 'error':
            return __assign({}, state, { error: action.error, loading: false });
        case 'move':
            if (!action.snapshot) {
                return state;
            }
            return __assign({}, state, { value: moveChild(state.value, action.snapshot, action.previousKey) });
        case 'remove':
            if (!action.snapshot) {
                return state;
            }
            return __assign({}, state, { value: removeChild(state.value, action.snapshot) });
        case 'reset':
            return initialState;
        case 'value':
            return __assign({}, state, { loading: false });
        default:
            return state;
    }
};
var useList = (function (query) {
    var _a = useReducer(reducer$1, initialState), state = _a[0], dispatch = _a[1];
    var ref = useIsEqualRef(query, function () { return dispatch({ type: 'reset' }); });
    var onChildAdded = function (snapshot, previousKey) {
        dispatch({ type: 'add', previousKey: previousKey, snapshot: snapshot });
    };
    var onChildChanged = function (snapshot) {
        dispatch({ type: 'change', snapshot: snapshot });
    };
    var onChildMoved = function (snapshot, previousKey) {
        dispatch({ type: 'move', previousKey: previousKey, snapshot: snapshot });
    };
    var onChildRemoved = function (snapshot) {
        dispatch({ type: 'remove', snapshot: snapshot });
    };
    useEffect(function () {
        var query = ref.current;
        if (!query) {
            dispatch({ type: 'value' });
            return;
        }
        // This is here to indicate that all the data has been successfully received
        query.once('value', function () {
            dispatch({ type: 'value' });
        }, function (error) {
            dispatch({ type: 'error', error: error });
        });
        query.on('child_added', onChildAdded);
        query.on('child_changed', onChildChanged);
        query.on('child_moved', onChildMoved);
        query.on('child_removed', onChildRemoved);
        return function () {
            query.off('child_added', onChildAdded);
            query.off('child_changed', onChildChanged);
            query.off('child_moved', onChildMoved);
            query.off('child_removed', onChildRemoved);
        };
    }, [ref.current]);
    return {
        error: state.error,
        loading: state.loading,
        value: state.value.values,
    };
});
var addChild = function (currentState, snapshot, previousKey) {
    if (!snapshot.key) {
        return currentState;
    }
    var keys = currentState.keys, values = currentState.values;
    if (!previousKey) {
        // The child has been added to the start of the list
        return {
            keys: [snapshot.key].concat(keys),
            values: [snapshot].concat(values),
        };
    }
    // Establish the index for the previous child in the list
    var index = keys.indexOf(previousKey);
    // Insert the item after the previous child
    return {
        keys: keys.slice(0, index + 1).concat([snapshot.key], keys.slice(index + 1)),
        values: values.slice(0, index + 1).concat([
            snapshot
        ], values.slice(index + 1)),
    };
};
var changeChild = function (currentState, snapshot) {
    if (!snapshot.key) {
        return currentState;
    }
    var index = currentState.keys.indexOf(snapshot.key);
    return __assign({}, currentState, { values: currentState.values.slice(0, index).concat([
            snapshot
        ], currentState.values.slice(index + 1)) });
};
var removeChild = function (currentState, snapshot) {
    if (!snapshot.key) {
        return currentState;
    }
    var keys = currentState.keys, values = currentState.values;
    var index = keys.indexOf(snapshot.key);
    return {
        keys: keys.slice(0, index).concat(keys.slice(index + 1)),
        values: values.slice(0, index).concat(values.slice(index + 1)),
    };
};
var moveChild = function (currentState, snapshot, previousKey) {
    // Remove the child from it's previous location
    var tempValue = removeChild(currentState, snapshot);
    // Add the child into it's new location
    return addChild(tempValue, snapshot, previousKey);
};

var useListKeys = (function (query) {
    var _a = useList(query), error = _a.error, loading = _a.loading, value = _a.value;
    return {
        error: error,
        loading: loading,
        value: value.map(function (snapshot) { return snapshot.key; }),
    };
});

var useListVals = (function (query, keyField) {
    var _a = useList(query), error = _a.error, loading = _a.loading, value = _a.value;
    return {
        error: error,
        loading: loading,
        value: value.map(function (snapshot) {
            return keyField
                ? __assign({}, snapshot.val(), (_a = {}, _a[keyField] = snapshot.key, _a)) : snapshot.val();
            var _a;
        }),
    };
});

var useObject = (function (query) {
    var _a = useLoadingValue(), error = _a.error, loading = _a.loading, reset = _a.reset, setError = _a.setError, setValue = _a.setValue, value = _a.value;
    var ref = useIsEqualRef(query, reset);
    useEffect(function () {
        var query = ref.current;
        if (!query) {
            setValue(null);
            return;
        }
        query.on('value', setValue, setError);
        return function () {
            query.off('value', setValue);
        };
    }, [ref.current]);
    return {
        error: error,
        loading: loading,
        value: value,
    };
});

var useObjectVal = (function (query) {
    var _a = useObject(query), error = _a.error, loading = _a.loading, value = _a.value;
    return {
        error: error,
        loading: loading,
        value: value ? value.val() : undefined,
    };
});

export { useList, useListKeys, useListVals, useObject, useObjectVal };
